#ifndef BDMFLASH_H
#define BDMFLASH_H

#include <sys/types.h>

#define FLASH_ALG_BITS_x8    0
#define FLASH_ALG_BITS_x16   1
#define FLASH_ALG_BITS_x8x2  2

/* Structure describing programming operations for flash type */
typedef struct flash_alg_info {
  /* Sets retid to manufacturer and type ID, returns <0 in case of error */
  int (*check_id)(struct flash_alg_info const *alg, void *addr, u_int16_t retid[2]);
  /* Programs one location of flash (16 bits for width=1 or 2; 8 bits for width8=0) */
  int (*prog)(struct flash_alg_info const *alg, void *addr, u_int16_t val);
  /* Erase all sectors overlaped by region from addr of size bytes, size=0 => erase all */
  /* This version is capable only of full erase (size=0) and one sector (size=1) */
  int (*erase)(struct flash_alg_info const *alg, void *addr, long size);
  /* Numeric and string fields follows */
  u_int32_t addr_mask;	/* Mask to take offset inside flash */
  u_int32_t reg1_addr;	/* Flash control register 1 */
  u_int32_t reg2_addr;	/* Flash control register 2 */
  u_int32_t sec_size;	/* block size of bigger blocks */
  u_int16_t width;	/* FLASH_ALG_BITS_x8 .. 8 bit data bus,
			   FLASH_ALG_BITS_x16 .. 16 bit data,
  			   FLASH_ALG_BITS_x8x2 .. two interleaved 8 bit */
  u_int16_t cmd_unlock1;/* first byte of command sequence */
  u_int16_t cmd_unlock2;/* second byte of command sequence */
  u_int16_t cmd_rdid;	/* read identifier */
  u_int16_t cmd_prog;	/* program one loc */
  u_int16_t cmd_erase;	/* erase command */
  u_int16_t cmd_reset;	/* leave program mode */
  u_int16_t erase_all;	/* erase all */
  u_int16_t erase_sec;	/* erase sector */
  u_int16_t fault_bit;	/* programing of location failed */
  u_int16_t manid;	/* manufacturer ID */
  u_int16_t devid;	/* device ID */
  char *alg_name;	/* informative flash type name */
} flash_alg_info_t;

int bdmflash_check_id(flash_alg_info_t const *alg, void *addr,
		   u_int16_t retid[2]);

int bdmflash_prog(flash_alg_info_t const *alg, void *addr, u_int16_t val);

int bdmflash_erase(flash_alg_info_t const *alg, void *addr, long size);

flash_alg_info_t **flash_alg_infos;

flash_alg_info_t const *bdmflash_alg_from_id(u_int16_t id[2]);

flash_alg_info_t const *bdmflash_alg_probe(caddr_t flash_adr);

int bdmflash_wrb_filt(bdmlib_bfilt_t * filt, caddr_t in_adr,
		u_int size, u_char * bl_ptr);

int bdmflash_erase_filt(bdmlib_bfilt_t * filt, caddr_t in_adr, u_int size);

int bdmflash_blankck_filt(bdmlib_bfilt_t * filt, caddr_t in_adr, u_int size);

#endif /* BDMFLASH_H */
